// Generate a model of the grid visible on the Prusa i3 MK2 bed
// This script generates a model, which when sliced draws a single 
// filament width on the major grid lines.  This helps to visualize
// how the coordinate system used by the printer & fix any issues.
//
// Slicing instructions:
//   Disable skirt & brim.  The grid generated by this bed fills all
//   the print area.  In Slic3r, I do this by setting loops to 0, and
//   minimum extrusion length to 0.
//
// Printing instructions:
//   Use a filament of colour that contrasts with the bed for best 
//   results. I have used white, it works great.

// Configurable parameters

// Paramaters below are tuned for a 0.4 mm nozzle.  Tuning with initial 
// parameters may be required to get exact results with other nozzles.  
// Fine nozzles (e.g. 0.25 mm) may provide better estimates of perfection.

fW=0.55; // filament width - 0.55 generates all layers
// Filament width less than 0.42 does not generate any g-code on slicing
// Width 0.5 does not generate g-code for x=0 line.
layerHeight=0.2;
numLayers=2;

// Logic starts here
//

hfW=fW*0.5;

// Coordinates for grid points.  Basically first & last
// lines are drawn on the coordinates. Rest are drawn
// centered on the coordinate.
xNext= [50-hfW,  100-hfW,   150-hfW,  200-hfW,250-fW];
xPrev=[fW, 50+hfW, 100+hfW,150+hfW, 200+hfW];
    
yNext = [55-hfW, 105-hfW, 155-hfW, 205-hfW, 210-fW];
yPrev = [fW, 55+hfW, 105+hfW, 155+hfW, 205+hfW];

// Generate the geometry.  This could be done using
// a single polygon with a hole. I find the difference
// approach below easier to understand & implement.
linear_extrude(layerHeight*numLayers)
difference() {
    polygon([[0,0], [250,0], [250,210], [0,210], [0,0]]);
    
    for(iy=[0:4]) {
       for(ix=[0:4]) {
           x1 = xPrev[ix];
           y1 = yPrev[iy];
           x2 = xNext[ix];
           y2 = yNext[iy];
           polygon([[x1,y1],[x2,y1],[x2,y2],[x1,y2],[x1,y1]]);
           echo(x1, y1, x2, y2);
       }
   } 
}